<?php
namespace Teknisa\CMP\Modules\Fornecimento\Modules\AutorizacaoFornecimento\Services; 

use Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use Teknisa\CMP\Util\CmpUtilities;

class Cmp07600AlteracaoAprovacaoAfService {

    protected $connection;
    protected $smtpUtil;
    protected $entityManager;
    protected $cmpUtilities;
    
    public function __construct ($entityManager, $smtpUtil, CmpUtilities $cmpUtilities) {
        $this->smtpUtil = $smtpUtil; 
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->cmpUtilities = $cmpUtilities;
    }
    
    public function getSystemParamGeral(){
        
        $select = "SELECT IDSISDOS FROM PARAMGERAL";

        try
        {
            $std = $this->connection->prepare($select);
            $std->execute();
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getBranchOperator($selectParams){
        
        $select = "SELECT DISTINCT CDOPERADOR
                      FROM GRUPFILIOPER, FILGRUPFILI
                     WHERE GRUPFILIOPER.CDGRUPFILI = FILGRUPFILI.CDGRUPFILI 
                       AND FILGRUPFILI.CDFILIAL    = :P_CDFILDEST";
        
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getTradingCenter($selectParams){
        
        $select = "SELECT NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) AS CDFILCENTCMP
                      FROM FILIAL FIL, FILIAL
                     WHERE FILIAL.CDFILIAL = :P_CDFILDEST
                       AND NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) = FIL.CDFILIAL";
        
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getPermissions($selectParams){
        
        $select = "SELECT CDSUBGRUPPROD, CDGRUPPROD, IDBLOQFORNCOTAF, IDCARPRECLAF
                      FROM PARASUPRI
                     WHERE CDFILIAL = :P_CDFILCENTCMP
                       AND CDSUBGRUPPROD IS NOT NULL";
        
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getIntegration($row, $selectParams){
        
        $select = self::getIntegrationSelectByCurrentSystem($row);
        
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getIntegrationSelectByCurrentSystem($row){
        
        if($row["IDSISDOS"] == 3){
            
            $select = "SELECT AUTOFORNE.CDFILDEST
                        FROM AUTOFORNE, FORNECEDOR
                       WHERE AUTOFORNE.CDFORNECED = FORNECEDOR.CDFORNECED
                        AND AUTOFORNE.CDFILDEST  = :P_CDFILDEST
                        AND AUTOFORNE.NRAUTOFORN = :P_NRAUTOFORN
                        AND NVL(FORNECEDOR.USR_INTEGRAMESAP, 'N') = 'S'
                        AND USR_IDINTEGRAMESAP IS NOT NULL";   
            
        } else {
            
            $select = "SELECT AUTOFORNE.CDFILDEST
                          FROM AUTOFORNE
                         WHERE AUTOFORNE.CDFILDEST  = :P_CDFILDEST
                           AND AUTOFORNE.NRAUTOFORN = :P_NRAUTOFORN";
            
        }
        
        return $select;
        
    }
    
    public function getFrequency($selectParams){
        
        $select = "SELECT PERIENTR.CDPERIENTR, PERIENTR.NMPERIENTR
                      FROM REFORFIL, PERIENTR
                     WHERE REFORFIL.CDPERIENTR = PERIENTR.CDPERIENTR
                       AND REFORFIL.CDFORNECED = :P_CDFORNECED
                       AND REFORFIL.CDFILIAL   = :P_CDFILDEST";
                       
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function executeProcedureCarregaPeriforn($row) {
       
        $procedure = new StoredProcedure($this->connection, "CARREGA_PERIFORN");
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row["CDFILDEST"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFORNECED', Param::PARAM_INPUT, $row["CDFORNECED"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTPROGENTAF', Param::PARAM_INPUT, $row["DTNOVAENTRAF"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NEW_DTPROGENTAF', Param::PARAM_OUTPUT, null, null, 20));
        
        $newDate = $procedure->execute();
        return $newDate["P_NEW_DTPROGENTAF"];
        
    }
    
    public function sendEmailToProductResponsibleOperators($row) {
        $attrImg = 'width="200px"';
	    $srcLogo = $this->cmpUtilities->returnLogo('PORTAL E-COMMERCE');
        $this->cmpUtilities->getLogoAttributes($srcLogo, $attrImg);
        $lgpd = $this->cmpUtilities->returnLgpd($row['FILIAL_LOGADA']);
        
        $subject = 'Aviso de Alteração de AF';
        $assunto = "<p class='bold' style='text-align: center; color: #ED2424'>Aviso de Alteração de AF</p>";
        
        foreach ($row["OPERADORES_RESPONSAVEIS"] as $operator) {
            try {
                $mensagem = "";
                $mensagem .= "<p>" . $operator["NMOPERADOR"] . ", </p>";
                $mensagem .= "<p> Abaixo seguem as informações da autorização do fornecimento cuja alteração deve ser analisada por você. São elas: </p>";
                $mensagem .= "<ul>
                                <li><strong>AF: </strong>" . $row["NRAUTOFORN"] . "</li>
                                <li><strong>Unidade Lançamento: </strong>" . $row["CDFILLANC"] . "</li>
                                <li><strong>Unidade Destino: </strong>" . $row["NMFILDEST"] . "</li>
                                <li><strong>Data de Entrega: </strong>" . $row["DTPROGENTAF"] . "</li>
                                <li><strong>Comprador: </strong>" . $row["NMOPERADOR"] . "</li>
                                <li><strong>Fornecedor: </strong>" . $row["NMRAZSOCFORN"] . "</li>
                              </ul>";
                              
                $messageSendMail = $this->cmpUtilities->returnMessageSendMail($srcLogo, $assunto, $mensagem, $attrImg, $lgpd);
                $this->cmpUtilities->enviaEmail($row['EMAIL_LOGADO'], $subject, $operator["DSEMAILOPER"], $messageSendMail, $operator["DSEMAILOPER"]);
            } catch(\Exception $e){
                throw $e;
            }
        }
    }
    
    public function getAfResponsibleOperators($selectParams){
        
        $select = "SELECT DISTINCT OPERADOR.CDOPERADOR, OPERADOR.DSEMAILOPER, OPERADOR.CDSENEMAILOP, OPERADOR.NMOPERADOR
                      FROM GRUPFILIOPER, FILGRUPFILI, OPERADOR
                     WHERE GRUPFILIOPER.CDGRUPFILI = FILGRUPFILI.CDGRUPFILI 
                       AND FILGRUPFILI.CDFILIAL    = :P_CDFILDEST
                       AND GRUPFILIOPER.CDOPERADOR = OPERADOR.CDOPERADOR";
       
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function updateAllAfProductDeliveryDate($updateParams){
        
        $update = "UPDATE ITEMAUTF
                    SET DTNOVAENTRAF = :P_DTNOVAENTRAF
                   WHERE CDFILDEST = :P_CDFILDEST
                   AND CDFILLANC = :P_CDFILLANC
                   AND NRAUTOFORN = :P_NRAUTOFORN";
        
        try
        {
            $std = $this->connection->prepare($update);
            $std->execute($updateParams);
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getProductsWithNewDeliveryDate($selectParams){
        
        $select = "SELECT QTNOVAPROGAF
                    FROM ITEMAUTF    
                   WHERE CDFILDEST = :P_CDFILDEST
                   AND CDFILLANC = :P_CDFILLANC
                   AND NRAUTOFORN = :P_NRAUTOFORN
                   AND QTNOVAPROGAF IS NOT NULL";
        
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getParcmpsup($selectParams){
        
        $select = "SELECT IDOBLIBAFMAN
                      FROM PARCMPSUP
                     WHERE CDFILIAL = :CDFILIAL
                       AND NRORG = :NRORG";
        
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getParamSupplier($params){
        
        $query = "SELECT
                     AUTOFORNE.CDFILDEST,
                     NVL(FORNECEDOR.USR_INTEGRAMESAP,'N') USR_INTEGRAMESAP,
                     FORNECEDOR.IDPERALTAFCONF,
                     NVL(AUTOFORNE.IDSITUATENDIMENTOAF,'X') IDSITUATENDIMENTOAF
                 FROM
                     AUTOFORNE,
                     FORNECEDOR
                 WHERE
                     AUTOFORNE.CDFORNECED = FORNECEDOR.CDFORNECED
                     AND AUTOFORNE.CDFILDEST =:P_CDFILDEST
                     AND AUTOFORNE.NRAUTOFORN =:P_NRAUTOFORN";

        $std = $this->connection->prepare($query);
        $std->execute($params);
        return $std->fetchAll()[0];
        
    }
    
    public function updateAf($updateParams){
        
          $update = "UPDATE AUTOFORNE
                    SET IDAFFORALIM = :IDAFFORALIM,
                        IDSITUATUAAF = :IDSITUATUAAF "
                    . $updateParams['CDGERENTE'] .    
                   "WHERE CDFILDEST = :CDFILDEST
                   AND CDFILLANC = :CDFILLANC
                   AND NRAUTOFORN = :NRAUTOFORN
                   AND NRORG = :NRORG";
        
        $params = array(
            
            "IDAFFORALIM"   => $updateParams["IDAFFORALIM"],
            "IDSITUATUAAF"  => $updateParams["IDSITUATUAAF"],
            "CDFILDEST"     => $updateParams["CDFILDEST"],
            "CDFILLANC"     => $updateParams["CDFILLANC"],
            "NRAUTOFORN"    => $updateParams["NRAUTOFORN"],
            "NRORG"         => $updateParams["NRORG"]
            
        );
        
        try
        {
            $std = $this->connection->prepare($update);
            $std->execute($params);
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function updateCdtpctpagar($updateParams) {
        $query = "UPDATE AUTOFORNE
            SET CDTPCTPAGAR =
              (SELECT PRODUTO.CDTPCTPAGAR
              FROM PRODUTO,
                (SELECT MAX(PRODUTO.CDARVPROD) AS CDARVPROD
                FROM PRODUTO,
                  (SELECT MIN(A.CDARVPROD) AS CDARVPROD
                  FROM
                    (SELECT CDARVPROD
                    FROM ITEMAUTF,
                      PRODUTO P
                    WHERE P.CDPRODUTO       = ITEMAUTF.CDPRODUTO
                    AND ITEMAUTF.CDFILDEST  = :CDFILDEST
                    AND ITEMAUTF.CDFILLANC  = :CDFILLANC
                    AND ITEMAUTF.NRAUTOFORN = :NRAUTOFORN
                    AND ITEMAUTF.NRORG = :NRORG
                    GROUP BY CDARVPROD
                    HAVING MAX(QTPROGENTAF   *VRPREUNIAF) =
                      (SELECT MAX(QTPROGENTAF*VRPREUNIAF)
                      FROM ITEMAUTF
                      WHERE ITEMAUTF.CDFILDEST = :CDFILDEST
                      AND ITEMAUTF.CDFILLANC   = :CDFILLANC
                      AND ITEMAUTF.NRAUTOFORN  = :NRAUTOFORN
                      AND ITEMAUTF.NRORG = :NRORG
                      )
                    )A
                  ) ITEMAUTF
                WHERE ITEMAUTF.CDARVPROD LIKE PRODUTO.CDARVPROD
                  || '%'
                AND PRODUTO.CDTPCTPAGAR IS NOT NULL
                AND PRODUTO.NRORG = :NRORG
                ) TIPOCTPAGAR
              WHERE PRODUTO.CDARVPROD  = TIPOCTPAGAR.CDARVPROD
              AND PRODUTO.CDTPCTPAGAR IS NOT NULL
              AND PRODUTO.NRORG = :NRORG
              )
            WHERE AUTOFORNE.CDFILDEST  = :CDFILDEST
            AND AUTOFORNE.CDFILLANC    = :CDFILLANC
            AND AUTOFORNE.NRAUTOFORN   = :NRAUTOFORN
            AND AUTOFORNE.NRORG = :NRORG
            AND AUTOFORNE.CDTPCTPAGAR IS NULL";
            
        $params = array(
            "CDFILDEST"  => $updateParams["CDFILDEST"],
            "CDFILLANC"  => $updateParams["CDFILLANC"],
            "NRAUTOFORN" => $updateParams["NRAUTOFORN"],
            "NRORG"      => $updateParams["NRORG"]
        );
        $result = $this->entityManager->getConnection()->executeUpdate($query, $params);
        return $result;
    }
        
    public function CUSTO_AF($procParams) {
        $connection = $this->entityManager->getConnection();
        $procedure = new StoredProcedure($connection, 'CUSTO_AF');
        $procedure->addParam(new Param('P_CDFILDEST',   Param::PARAM_INPUT, $procParams["CDFILDEST"],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILLANC',   Param::PARAM_INPUT, $procParams["CDFILLANC"],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAUTOFORN',  Param::PARAM_INPUT, $procParams["NRAUTOFORN"],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSEQAPURAF', Param::PARAM_INPUT, $procParams["NRSEQAPURAF"], Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
    
    public function IMPOSTO_ITEMAF($row) {
        $connection = $this->entityManager->getConnection();
        $procedure = new StoredProcedure($connection, 'IMPOSTO_ITEMAF');
        $procedure->addParam(new Param('P_EMPRESA',    Param::PARAM_INPUT, $row["CDEMPRESA"],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INSCESTA',   Param::PARAM_INPUT, $row["CDINSCESTA"],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FORNECEDOR', Param::PARAM_INPUT, $row["CDFORNECED"],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILDEST',    Param::PARAM_INPUT, $row["CDFILDEST"],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILLANC',    Param::PARAM_INPUT, $row["CDFILLANC"],   Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param('P_LANCTOAF',   Param::PARAM_INPUT, $row["NRAUTOFORN"],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INIVALID',   Param::PARAM_INPUT, $row["DTPROGENTAF"], Param::PARAM_TYPE_STR));
        $procedure->execute();
    }

}