<?php
namespace Folha\Logic\ESocial\Eventos\Data;

use Folha\Logic\ESocial\GerenciaEnvios;

/**
 * Classe provedora do datasource utilizado para a geracao do evento S-1010 - Tabela de Rubricas
 *
 * @package Folha\Logic\ESocial\Eventos\Data
 * @author paulopereira
 **/
class S1010
{

    /**
     * Retorna os dados que serao utilizado para a geracao do(s) arquivo(s) do evento S-1010, Tabela de Rubricas
     *
     * @param $dsordem                 Define as possíveis formas de filtro utilizadas pela tela de envio de eventos  
     *                                 do e-Social. Pode ser ALFABETICA, MATRICULA, DATA_ADMISSAO e ESTRUTURA. Para este
     *                                 evento, em especial, somente o filtro por ESTRUTURA poderá será relevante.
     * @param $limiteInferior          Limite inferior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $limiteSuperior          Limite superior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $inicioPeriodo           Inicio do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true". 
     * @param $fimPeriodo              Final do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true".
     * @param $utilizaIntervaloDeDatas Verifica se o dataset deve ser filtrado pelos campos de intervalo da tela de  
     *                                 envio de eventos e-Social, ou se devem enviados os arquivos alterados depois do 
     *                                 ultimo envio.
     * @param $nrorg                   Nrorg.
     * @param $versaoHRM               Versao do sistema HRM utilizado para a geracao dos relatorios.
     *
     * @return array
     * @author paulopereira
     **/
    public static function getDataSource($dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo,
                                         $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $versaoEsocial, $tpAmbiente)
    {
        $pastaLog = "C:\\LogESocial";
        if (!is_dir($pastaLog)) {
            mkdir($pastaLog, 0777, true);
        }

        $filename = "$pastaLog\\LogS1010.txt";
        $log = fopen($filename,"w+");

        $dateTimeIni = new \DateTime();
        fwrite($log, "[".$dateTimeIni->format('d/m/Y H:i:s')."]Gerando rubricas...\n");

        $S1010 = \Folha\Repositories\GpeEsocialRegistro::dataSetEsocial_S1010(
            $dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo,
            $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $versaoEsocial, $tpAmbiente
        );

        $dateTime = new \DateTime();
        fwrite($log, "[".$dateTime->format('d/m/Y H:i:s')."]Rubricas geradas...\n");
        foreach ($S1010 as &$row) {
            //todo: Tratar os casos de eventos de exclusão dos cadastros

            /*Verifica pelo log de envios (GPE_ESOCIAL_ENVIO) se o evento/chave já foi enviado para definir se é um envio de inclusão ou alteração*/
            $operacao = new GerenciaEnvios('S-1010', $versaoEsocial, $row['CHAVE'], null, $nrorg);
            $operacao = $operacao->validaTipoEnvio();
            $row['INCLUSAO'] = $operacao['INCLUSAO'];
            $row['ALTERACAO'] = $operacao['ALTERACAO'];

            $dateTime = new \DateTime();
            fwrite($log, "[".$dateTime->format('d/m/Y H:i:s')."]Log rubrica ".$row['CHAVE']." gerado...\n");

            $row['CODINCCP'] = self::retornaIncidCP($row['CHAVE']);
            $row['CODINCIRRF'] = self::retornaIncidIRRF($row['CHAVE']);
            $row['CODINCFGTS'] = self::retornaIncidFGTS($row['CHAVE']);
            $row['CODINCSIND'] = self::retornaIncidSIND($row['CHAVE']);

            $dateTime = new \DateTime();
            fwrite($log, "[".$dateTime->format('d/m/Y H:i:s')."]Retornadas incidências da rubrica ".$row['CHAVE']."...\n");

            /*Verifica os tipos de incidência para o evento*/
            /*Aqui é retornada uma string com os tipos de incidência para o evento.*/
            $listTipoincide = \Folha\Repositories\FpaIncidevento::retornaListaTipoIncideEvento($row['CHAVE'], $nrorg);
            $row['REPDSR'] = self::retornaIncidDSR($row['CHAVE']);
            $row['REP13'] = self::retornaIncid13($listTipoincide);
            $row['REPFERIAS'] = self::retornaIncidFerias($listTipoincide);
            $row['REPRESC'] = self::retornaIncidRescisao($listTipoincide);

            $dateTime = new \DateTime();
            fwrite($log, "[".$dateTime->format('d/m/Y H:i:s')."]Retornadas repercussões da rubrica ".$row['CHAVE']."...\n");
        }

        $dateTimeFim = new \DateTime();
        $dateTimeDif = $dateTimeIni->diff($dateTimeFim);
        fwrite($log, "[".$dateTimeFim->format('d/m/Y H:i:s')."]Finalizada geração das rubricas em ".$dateTimeDif->format('%H:%I:%S hours')."!!!\n");
        tk_var_dump($dateTimeDif);

        fclose($log);

        return $S1010;
    }

    /**
     * Incidência tributária da rubrica para a Previdência Social
     */
    private function retornaIncidCP($nrevento)
    {
        switch ($nrevento) {
            /*11 - Base de Cálculo (BC) do Salário de Contribuição (SC) Mensal*/
            case \FpatipoassoceveEntity::ASSOC_INCID_INSS:
                $codIncid = '11'; break;
            /*12 - Base de Cálculo da Contrib. Prev. sobre 13o Salário*/
            case \FpatipoassoceveEntity::ASSOC_INCID_INSS_13SAL:
                $codIncid = '12'; break;
            /*21 - Base de Cálculo da Contrib. Prev. Sal. Maternidade*/
            //
            /*22 - Base de Cálculo da Contrib. Prev. Sal. Maternidade - 13o Salário*/
            //
            /*23 - Base de Cálculo da Contrib. Prev. Auxilio Doença - Regime Próprio de Previdência Social*/
            //
            /*24 - Base de Cálculo da Contrib. Prev. Auxilio Doença - 13o Salário Doença - Regime Próprio de Previdência Social*/
            //
            /*31 - Contribuição Descontada do Segurado - SC Mensal*/
            case \FpatipoassoceveEntity::ASSOC_INSS_JA_DESC_CALC:
                $codIncid = '31'; break;
            /*32 - Contribuição Descontada do Segurado - 13o Salário*/
            case \FpatipoassoceveEntity::ASSOC_INSS_JA_DESC_13SAL:
                $codIncid = '32'; break;
            /*34 - Contribuição Descontada do Segurado - SEST + SENAT*/
            //
            /*51 - Salário-Família*/
            //
            /*61 - Complemento de salário-mínimo - Regime Próprio de Previdência Social*/
            //
            /*91 - Incid. suspensa em decorrência de decisão judicial ou consulta interna RFB (BC SC Mensal)*/
            //
            /*92 - Incid. suspensa em decorrência de decisão judicial ou consulta interna RFB (BC 13o Salário)*/
            //
            /*93 - Incid. suspensa em decorrência de decisão judicial ou consulta interna RFB (BC Sal. Maternidade)*/
            //
            /*94 - Incid. suspensa em decorrência de decisão judicial ou consulta interna RFB (BC Sal. Maternidade 13o Salário)*/
            //
            /*00 - Não é Base de Cálculo*/
            default: $codIncid = '00'; break;
        }

        return $codIncid;
    }

    /**
     * Incidência tributária da rubrica para o IRRF
     */
    private function retornaIncidIRRF($nrevento)
    {
        switch ($nrevento) {
            //BASES DE CÁLCULO:
            /*11 - Base de cálculo do IRRF - Remuneração Mensal*/
            case \FpatipoassoceveEntity::ASSOC_INCID_IRRF:
                $codIncid = '11'; break;
            /*12 - Base de cálculo do IRRF - 13o Salário*/
            case \FpatipoassoceveEntity::ASSOC_INCID_IRRF_13SAL:
                $codIncid = '12'; break;
            /*13 - Base de cálculo do IRRF - Férias*/
            case \FpatipoassoceveEntity::ASSOC_INCID_IRRF_FERIAS:
                $codIncid = '13'; break;
            /*14 - Base de cálculo do IRRF - PLR*/
            //$codIncid = '11'; break;
            /*15 - Base de cálculo do IRRF - Rendimentos Recebidos Acumuladamente - RRA*/
            //
            //VALOR DO DESCONTO DO IRRF:
            /*31 - Valor do IRRF - Remuneração Mensal*/
            case \FpatipoassoceveEntity::ASSOC_IRRF_CALC:
                $codIncid = '31'; break;
            /*32 - Valor do IRRF - 13o Salário*/
            case \FpatipoassoceveEntity::ASSOC_IRRF_13SAL:
                $codIncid = '32'; break;
            /*33 - Valor do IRRF - Férias*/
            case \FpatipoassoceveEntity::ASSOC_IRRF_FERIAS:
                $codIncid = '33'; break;
            /*34 - Valor do IRRF - PLR*/
            //
            /*35 - Valor do IRRF - RRA*/
            //
            //DEDUÇÕES DA BASE DO IRRF:
            /*41 - Previdência Social Oficial - Remuneração Mensal*/
            //
            /*42 - Previdência Social Oficial - 13° salário*/
            //
            /*43 - Previdência Social Oficial - Férias*/
            //
            /*44 - Previdência Social Oficial - RRA*/
            //
            /*46 - Previdência Privada - salário mensal*/
            //
            /*47 - Previdência Privada - 13° salário*/
            //
            /*51 - Pensão Alimentícia - Remuneração Mensal*/
            //
            /*52 - Pensão Alimentícia - 13° salário*/
            //
            /*53 - Pensão Alimentícia - Férias*/
            //
            /*54 - Pensão Alimentícia - RRA*/
            //
            /*56 - Dependente - Remuneração Mensal*/
            //
            /*57 - Dependente - 13° salário*/
            //
            /*58 - Dependente - Férias*/
            //
            /*61 - Fundo de Aposentadoria Programada Individual - FAPI - Remuneração Mensal*/
            //
            /*62 - Fundo de Aposentadoria Programada Individual - FAPI - 13° salário*/
            //
            /*63 - Fundação de Previdência Complementar do Servidor Público Federal - Funpresp - Remuneração Mensal*/
            //
            /*64 - Fundação de Previdência Complementar do Servidor Público Federal - Funpresp - 13° salário*/

            //ISENÇÕES DO IRRF:
            /*71 - Parcela Isenta 65 anos - Remuneração Mensal*/
            //
            /*72 - Parcela Isenta 65 anos - 13° salário*/
            //
            /*73 - Diária e Ajuda de Custo*/
            //
            /*74 - Indenização e rescisão de contrato, inclusive a título de PDV*/
            //
            /*75 - Abono pecuniário*/
            //
            /*76 - Pensão, aposentadoria ou reforma por moléstia grave ou acidente em serviço - Remuneração Mensal*/
            //
            /*77 - Pensão, aposentadoria ou reforma por moléstia grave ou acidente em serviço - 13° salário*/
            //
            /*78 - Valores pagos a titular ou sócio de microempresa ou empresa de pequeno porte, exceto pró-labore e alugueis;*/
            //
            /*79 - Outros (o nome da rubrica deve ser claro para identificação)*/
            //
            /*81 - Depósito Judicial*/
            //
            /*82 - Compensação Judicial do ano calendário*/
            //
            /*83 - Compensação Judicial de anos anteriores*/
            //
            //INCIDÊNCIA SUSPENSA:
            /*91 - Incidência suspensa em decorrência de decisão judicial ou Consulta Interna RFB (BC IRRF Remuneração Mensal)*/
            //
            /*92 - Incidência suspensa em decorrência de decisão judicial ou Consulta Interna RFB (BC IRRF 13o Salário)*/
            //
            /*93 - Incidência suspensa em decorrência de decisão judicial ou Consulta Interna RFB (BC IRRF Férias)*/
            //
            /*94 - Incidência suspensa em decorrência de decisão judicial ou Consulta Interna RFB (BC IRRF PLR)*/
            //
            /*95 - Incidência suspensa em decorrência de decisão judicial ou Consulta Interna RFB (BC IRRF RRA)*/
            //
            /*00 - Não é Base de Cálculo*/
            default: $codIncid = '00';
        }

        return $codIncid;
    }

    /**
     * Incidência tributária da rubrica para o FGTS
     */
    private function retornaIncidFGTS($nrevento)
    {
        switch ($nrevento) {
            /*11 - Base de Cálculo do FGTS*/
            case \FpatipoassoceveEntity::ASSOC_INCID_FGTS:
                $codIncid = '11'; break;
            /*91 - Incidência suspensa em decorrência de decisão judicial*/
            //
            /*00 - Não é Base de Cálculo do FGTS*/
            default: $codIncid = '00';
        }

        return $codIncid;
    }

    /**
     * Incidência tributária da rubrica para a Contribuição Sindical Laboral
     */
    private function retornaIncidSIND($nrevento)
    {
        switch ($nrevento) {
            /*11 - Base de cálculo*/
            //
            /*31 - Valor da contribuição sindical laboral descontada*/
            case \FpatipoassoceveEntity::ASSOC_CONTRIB_SINDICAL:
                $codIncid = '31'; break;
            /*91 - Incidência suspensa em decorrência de decisão judicial*/
            //
            /*00 - Não é Base de Cálculo*/
            default: $codIncid = '00';
        }

            return $codIncid;
        }

        /**
         * Indica se a rubrica repercute no cálculo do Descanso Semanal
         */
    private function retornaIncidDSR($nrevento)
    {
        /*Repercute no cálculo do DSR*/
        if ($nrevento == '1040' or $nrevento == '1041') {
            $codIncid = 'S';
        }
        else {
            $codIncid = 'N';
        }

        return $codIncid;
    }

    /**
     * Indica se a rubrica repercute no cálculo do 13º Salário
     */
    private function retornaIncid13($listTipoincide)
    {
        /*Repercute no cálculo do 13º Salário*/
        if ((strpos($listTipoincide, '19') <> false) or
            (strpos($listTipoincide, '20') <> false) or
            (strpos($listTipoincide, '21') <> false) or
            (strpos($listTipoincide, '25') <> false)) {
            $codIncid = 'S';
        }
        else {
            $codIncid = 'N';
        }

        return $codIncid;
    }

    /**
     * Indica se a rubrica repercute no cálculo das Férias
     */
    private function retornaIncidFerias($listTipoincide)
    {
        /*Repercute no cálculo das Férias*/
        if ((strpos($listTipoincide, '16') <> false) or
            (strpos($listTipoincide, '17') <> false) or
            (strpos($listTipoincide, '18') <> false) or
            (strpos($listTipoincide, '24') <> false) or
            (strpos($listTipoincide, '37') <> false)) {
            $codIncid = 'S';
        }
        else {
            $codIncid = 'N';
        }

        return $codIncid;
    }

    /**
     * Indica se a rubrica repercute no cálculo da Rescisão
     */
    private function retornaIncidRescisao($listTipoincide)
    {
        /*Repercute no cálculo da Rescisão*/
        if ((strpos($listTipoincide, '4') <> false) or
            (strpos($listTipoincide, '5') <> false) or
            (strpos($listTipoincide, '6') <> false) or
            (strpos($listTipoincide, '7') <> false) or
            (strpos($listTipoincide, '8') <> false) or
            (strpos($listTipoincide, '9') <> false) or
            (strpos($listTipoincide, '10') <> false) or
            (strpos($listTipoincide, '11') <> false) or
            (strpos($listTipoincide, '12') <> false) or
            (strpos($listTipoincide, '67') <> false) or
            (strpos($listTipoincide, '68') <> false) or
            (strpos($listTipoincide, '69') <> false)) {
            $codIncid = 'S';
        }
        else {
            $codIncid = 'N';
        }

        return $codIncid;
    }

}